@@ -72,6 +72,9 @@ uint32_t buffer_get_int(Buffer *buffer);
 uint32_t get_u24(const void *vp);
 int buffer_get_int24_ret(uint32_t *ret, Buffer *buffer);
 uint32_t buffer_get_int24(Buffer *buffer);
+uint32_t get_u24le(const void *vp);
+int buffer_get_int24_le_ret(uint32_t *ret, Buffer *buffer);
+uint32_t buffer_get_int24_le(Buffer *buffer);
 uint64_t get_u64le(const void *vp);
 int buffer_get_int64_le_ret(uint64_t *ret, Buffer *buffer);
 uint64_t buffer_get_int64_le(Buffer *buffer);
